# -*- coding: utf-8 -*-

import uuid

from educommon.report import AbstractReportBuilder, AbstractDataProvider
from educommon.report.reporter import SimpleReporter, get_path, get_url

from kinder.core.declaration.models import Declaration


class BasePassingSmevReporter(SimpleReporter):

    u"""Базовый класс для отчетов - Миня СМЭВ."""

    def set_file_and_url(self):
        base_name = str(uuid.uuid4())[0:16] + self.extension
        self.out_file_path = get_path(base_name)
        self.out_file_url = get_url(base_name)
        return (self.out_file_path, self.out_file_url)


class FirstTypePassingSmevProvider(AbstractDataProvider):

    u"""Провайдер для первого типа отчета - Минуя СМЭВ."""

    def init(self, declaration_ids, report_name, **params):
        self.declaration_ids = declaration_ids
        self.report_name = report_name

    def get_declarations(self):
        delegate_lookup = (
            'declarationprivilege__privilegeconfirmationattributes__delegate'
        )
        return Declaration.objects.filter(
            pk__in=self.declaration_ids
        ).values_list(
            '{}__fullname'.format(delegate_lookup),
            '{}__date_of_birth'.format(delegate_lookup),
            '{}__reg_address_full'.format(delegate_lookup),
            '{}__snils'.format(delegate_lookup),
            'children__fullname',
            'children__date_of_birth',
        )


class FirstTypePassingSmevBuilder(AbstractReportBuilder):

    u"""Построитель для первого типа отчета - Минуя СМЭВ."""

    def __init__(self, provider, adapter, report, params):
        self.provider = provider
        self.report = report

    def _init_sections(self):
        get_section = self.report.get_section
        self.header_section = get_section('header')
        self.item_section = get_section('item')

    def build(self):
        self._init_sections()
        self.header_section.flush({'name': self.provider.report_name})

        for idx, (
            delegate_fullname,
            delegate_date_of_birth,
            delegate_address,
            delegate_snils,
            fullname,
            date_of_birth
        ) in enumerate(self.provider.get_declarations(), 1):
            self.item_section.flush({
                'idx': idx,
                'delegate_fullname': delegate_fullname,
                'delegate_date_of_birth': delegate_date_of_birth,
                'delegate_address': delegate_address,
                'delegate_snils': delegate_snils,
                'children_info': u', '.join((
                    fullname, date_of_birth.strftime('%d.%m.%Y')
                )),
            })


class FirstTypePassingSmevReporter(BasePassingSmevReporter):

    u"""Первый тип отчета - Минуя СМЭВ."""

    extension = '.xls'
    template_file_path = './templates/first_type'
    data_provider_class = FirstTypePassingSmevProvider
    builder_class = FirstTypePassingSmevBuilder
